// Learn TypeScript:
//  - https://docs.cocos.com/creator/manual/en/scripting/typescript.html
// Learn Attribute:
//  - https://docs.cocos.com/creator/manual/en/scripting/reference/attributes.html
// Learn life-cycle callbacks:
//  - https://docs.cocos.com/creator/manual/en/scripting/life-cycle-callbacks.html

import clientEvent from "./clientEvent";

const {ccclass, property} = cc._decorator;
@ccclass
export default class NewClass extends cc.Component {

    @property(cc.Node)
    target: cc.Node = null;
    rigidBody: cc.RigidBody;
    motor: cc.MotorJoint;
    collider: cc.PhysicsPolygonCollider;
    startPoint: cc.Vec2;
    tween: any;
    points:cc.Vec3;
    startCollision: boolean =false;
    relativeVelocity: any;
    dir: cc.Vec3;
    startMove: boolean = false;
    // LIFE-CYCLE CALLBACKS:
    onBeginContact(contact, selfCollider, otherCollider) {
        
    }
    onEndContact(contact, selfCollider, otherCollider) {
        
    }
    onPreSolve(contact, selfCollider, otherCollider) {
        if (otherCollider.node.name.includes('staticGear')) {
            // this.node.position = otherCollider.node.position;
            var worldManifold = contact.getWorldManifold();
            var vel1 = selfCollider.node.getComponent(cc.RigidBody).getLinearVelocityFromWorldPoint(worldManifold.points[0]);
            var vel2 = otherCollider.node.getComponent(cc.RigidBody).getLinearVelocityFromWorldPoint(worldManifold.points[0]);
            this.relativeVelocity = vel1.sub(vel2);
            this.scheduleOnce(()=>{
                this.rigidBody.linearVelocity = this.relativeVelocity;
                setTimeout(() => {
                    this.rigidBody.linearVelocity = cc.v2(0,0); 
                    this.rigidBody.angularVelocity = - otherCollider.node.getComponent(cc.RigidBody).angularVelocity;
                }, 100);
            }, 0);
            }
        }

        onPostSolve(contact, selfCollider, otherCollider) {
            this.rigidBody.angularVelocity = 0;
            // this.startCollision = false;
        }

    onLoad () {
        this.initEvent();
        this.rigidBody = this.node.getComponent(cc.RigidBody);
    }

    onEnable() {
    }

    initEvent() {
        this.node.on(cc.Node.EventType.TOUCH_START, this._touchStartEvent, this);
        this.node.on(cc.Node.EventType.TOUCH_MOVE, this._touchMoveEvent, this);
        this.node.on(cc.Node.EventType.TOUCH_END, this._touchEndEvent, this);
        this.node.on(cc.Node.EventType.TOUCH_CANCEL, this._touchCancelEvent, this);
    }

    _touchStartEvent(event: cc.Event.EventTouch) {
        this.startPoint = event.getStartLocation();
    }
    _touchMoveEvent(event: cc.Event.EventTouch) {
        var touches = event.getTouches();
        let newPos = this.node.parent.convertToNodeSpaceAR(touches[0].getLocation());
        // let subPos = oldPos.sub(newPos);
        // let dir = cc.v3(this.nodePos.x - subPos.x, this.nodePos.y - subPos.y, 0);
        this.dir = cc.v3(newPos.x, newPos.y, 0);
        this.startMove = true;
    }

    _touchEndEvent(event: cc.Event.EventTouch) {
       
    }

    _touchCancelEvent(event: cc.Event.EventTouch) {
        this.startMove = false;
    }

    start () {

    }

    update (dt) {
        if(!this.startMove){
            return;
        }
        const oldPos = cc.v3 (this.node.position.x, this.node.position.y, 0);
        let targetPos = this.dir.clone();
        let subPos = targetPos.sub(oldPos);
        let distance = subPos.mag();
        if (distance > 100) {
            targetPos =  oldPos.clone().add(targetPos.sub(oldPos.clone()).normalizeSelf().mul(100));
        }
        this.node.position = this.node.position.clone().lerp(targetPos, 4*dt);
    }

}
